@@ -4,8 +4,11 @@ from zope.component import queryUtility
 from zope.interface import implements
 
 from Products.Archetypes import PloneMessageFactory as _
-from Products.Archetypes.Field import *
-from Products.Archetypes.Widget import *
+from Products.Archetypes.Field import BooleanField, LinesField, TextField, \
+    StringField, DateTimeField
+from Products.Archetypes.Widget import (
+    BooleanWidget, TagsWidget, TextAreaWidget, StringWidget,
+    DatetimeWidget, SelectWidget, AjaxSelectWidget)
 from Products.Archetypes.Schema import Schema
 from Products.Archetypes.Schema import MetadataSchema
 from Products.Archetypes.interfaces import IExtensibleMetadata
@@ -20,18 +23,18 @@ from DateTime.DateTime import DateTime
 from App.class_init import InitializeClass
 from App.special_dtml import DTMLFile
 from Products.CMFCore import permissions
-from Products.CMFCore.utils  import getToolByName
-from Products.CMFDefault.utils import _dtmldir
+from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.log import log_deprecated
 from ComputedAttribute import ComputedAttribute
 
-_marker=[]
+_marker = []
 
 # http://www.zope.org/Collectors/CMF/325
 # http://www.zope.org/Collectors/CMF/476
 _zone = DateTime().timezone()
 
-FLOOR_DATE = DateTime(1000, 1) # always effective
-CEILING_DATE = DateTime(2500, 0) # never expires
+FLOOR_DATE = DateTime(1000, 1)  # always effective
+CEILING_DATE = DateTime(2500, 0)  # never expires
 
 # We import this conditionally, in order not to introduce a hard dependency
 try:
@@ -40,21 +43,17 @@ try:
 except ImportError:
     HAS_PLONE_I18N = False
 
-## MIXIN
+
+# MIXIN
 class ExtensibleMetadata(Persistence.Persistent):
-    """a replacement for CMFDefault.DublinCore.DefaultDublinCoreImpl
+    """ A DC metadata implementation for Plone Archetypes
     """
-    # XXX This is not completely true. We need to review this later
-    # and make sure it is true.
-    # Just so you know, the problem here is that Title
-    # is on BaseObject.schema, so it does implement IExtensibleMetadata
-    # as long as both are used together.
+
     implements(IExtensibleMetadata)
 
     security = ClassSecurityInfo()
 
-    schema = type = MetadataSchema(
-        (
+    schema = type = MetadataSchema((
         BooleanField(
             'allowDiscussion',
             accessor="isDiscussable",
@@ -67,33 +66,33 @@ class ExtensibleMetadata(Persistence.Persistent):
                         default=u'Allow comments'),
                 description=_(u'help_allow_comments',
                               default=u'If selected, users can add comments '
-                                       'to this item.')
-                ),
+                              'to this item.')
+            ),
         ),
         LinesField(
             'subject',
             multiValued=1,
             accessor="Subject",
             searchable=True,
-            widget=KeywordWidget(
+            widget=TagsWidget(
                 label=_(u'label_tags', default=u'Tags'),
                 description=_(u'help_tags',
                               default=u'Tags are commonly used for ad-hoc '
-                                       'organization of content.'),
-                ),
+                              'organization of content.'),
+            ),
         ),
         TextField(
             'description',
             default='',
             searchable=1,
             accessor="Description",
-            default_content_type = 'text/plain',
-            allowable_content_types = ('text/plain',),
+            default_content_type='text/plain',
+            allowable_content_types=('text/plain',),
             widget=TextAreaWidget(
                 label=_(u'label_description', default=u'Description'),
                 description=_(u'help_description',
                               default=u'Used in item listings and search results.'),
-                ),
+            ),
         ),
         # Location, also known as Coverage in the DC metadata standard, but we
         # keep the term Location here for historical reasons.
@@ -101,119 +100,122 @@ class ExtensibleMetadata(Persistence.Persistent):
             'location',
             # why no accessor? http://dev.plone.org/plone/ticket/6424
             searchable=True,
-            widget = StringWidget(
-                label = _(u'label_location', default=u'Location'),
+            widget=StringWidget(
+                label=_(u'label_location', default=u'Location'),
                 description=_(u'help_location_dc',
                               default=u'The geographical location associated with the item, if applicable.'),
-                ),
+            ),
         ),
         LinesField(
             'contributors',
             accessor="Contributors",
-            widget=LinesWidget(
+            widget=AjaxSelectWidget(
                 label=_(u'label_contributors', u'Contributors'),
                 description=_(u'help_contributors',
                               default=u"The names of people that have contributed "
-                                       "to this item. Each contributor should "
-                                       "be on a separate line."),
-                ),
+                              "to this item. Each contributor should "
+                              "be on a separate line."),
+                vocabulary="plone.app.vocabularies.Users"
+            ),
         ),
         LinesField(
             'creators',
             accessor="Creators",
-            widget=LinesWidget(
+            widget=AjaxSelectWidget(
                 label=_(u'label_creators', u'Creators'),
                 description=_(u'help_creators',
                               default=u"Persons responsible for creating the content of "
-                                       "this item. Please enter a list of user names, one "
-                                       "per line. The principal creator should come first."),
-                rows = 3
-                ),
+                              "this item. Please enter a list of user names, one "
+                              "per line. The principal creator should come first."),
+                vocabulary="plone.app.vocabularies.Users"
+            ),
         ),
         DateTimeField(
             'effectiveDate',
             mutator='setEffectiveDate',
-            languageIndependent = True,
-            widget=CalendarWidget(
+            languageIndependent=True,
+            widget=DatetimeWidget(
                 label=_(u'label_effective_date', u'Publishing Date'),
                 description=_(u'help_effective_date',
                               default=u"The date when the item will be published. If no "
-                                       "date is selected the item will be published immediately."),
-                ),
+                              "date is selected the item will be published immediately."),
+            ),
         ),
         DateTimeField(
             'expirationDate',
             mutator='setExpirationDate',
-            languageIndependent = True,
-            widget=CalendarWidget(
+            languageIndependent=True,
+            widget=DatetimeWidget(
                 label=_(u'label_expiration_date', u'Expiration Date'),
                 description=_(u'help_expiration_date',
                               default=u"The date when the item expires. This will automatically "
-                                       "make the item invisible for others at the given date. "
-                                       "If no date is chosen, it will never expire."),
-                ),
+                              "make the item invisible for others at the given date. "
+                              "If no date is chosen, it will never expire."),
+            ),
         ),
         StringField(
             'language',
             accessor="Language",
-            default = config.LANGUAGE_DEFAULT,
-            default_method = 'defaultLanguage',
+            default=config.LANGUAGE_DEFAULT,
+            default_method='defaultLanguage',
             vocabulary='languages',
-            widget=LanguageWidget(
+            widget=SelectWidget(
                 label=_(u'label_language', default=u'Language'),
-                ),
+            ),
         ),
         TextField(
             'rights',
             accessor="Rights",
             default_method='defaultRights',
+            allowable_content_types=('text/plain',),
             widget=TextAreaWidget(
                 label=_(u'label_copyrights', default=u'Rights'),
                 description=_(u'help_copyrights',
                               default=u'Copyright statement or other rights information on this item.'),
-                )),
-        )) + Schema((
+            )),
+    )) + Schema((
         # XXX change this to MetadataSchema in AT 1.4
         # Currently we want to stay backward compatible without migration
         # between beta versions so creation and modification date are using the
         # standard schema which leads to AttributeStorage
         DateTimeField(
-            'creation_date',
-            accessor='created',
-            mutator='setCreationDate',
-            default_method=DateTime,
-            languageIndependent=True,
-            isMetadata=True,
-            schemata='metadata',
-            generateMode='mVc',
-            widget=CalendarWidget(
-                label=_(u'label_creation_date', default=u'Creation Date'),
-                description=_(u'help_creation_date',
-                              default=u'Date this object was created'),
-                visible={'edit':'invisible', 'view':'invisible'}),
-        ),
+                'creation_date',
+                accessor='created',
+                mutator='setCreationDate',
+                default_method=DateTime,
+                languageIndependent=True,
+                isMetadata=True,
+                schemata='metadata',
+                generateMode='mVc',
+                widget=DatetimeWidget(
+                    label=_(u'label_creation_date', default=u'Creation Date'),
+                    description=_(u'help_creation_date',
+                                  default=u'Date this object was created'),
+                    visible={'edit': 'invisible', 'view': 'invisible'}),
+                ),
         DateTimeField(
             'modification_date',
             accessor='modified',
-            mutator = 'setModificationDate',
+            mutator='setModificationDate',
             default_method=DateTime,
             languageIndependent=True,
             isMetadata=True,
             schemata='metadata',
             generateMode='mVc',
-            widget=CalendarWidget(
-                label=_(u'label_modification_date',
-                        default=u'Modification Date'),
-                description=_(u'help_modification_date',
-                              default=u'Date this content was modified last'),
-                visible={'edit':'invisible', 'view':'invisible'}),
+            widget=DatetimeWidget(
+                    label=_(u'label_modification_date',
+                            default=u'Modification Date'),
+                    description=_(u'help_modification_date',
+                                  default=u'Date this content was modified last'),
+                    visible={'edit': 'invisible', 'view': 'invisible'}),
         ),
-        ))
+    ))
 
     def __init__(self):
         pass
 
     security.declarePrivate('defaultLanguage')
+
     def defaultLanguage(self):
         """Retrieve the default language"""
         tool = getToolByName(self, 'portal_languages', None)
@@ -222,12 +224,16 @@ class ExtensibleMetadata(Persistence.Persistent):
         return config.LANGUAGE_DEFAULT
 
     security.declarePrivate('defaultRights')
+
     def defaultRights(self):
         """Retrieve the default rights"""
         mdtool = getToolByName(self, 'portal_metadata', None)
         if mdtool is None:
             return ''
         for sid, schema in mdtool.listSchemas():
+            if not hasattr(schema, 'listPolicies'):
+                # Broken class from CMFDefault.
+                continue
             for pid, policy in schema.listPolicies(typ=self.Type()):
                 if pid != 'Rights' and not policy.supply_default:
                     continue
@@ -235,13 +241,33 @@ class ExtensibleMetadata(Persistence.Persistent):
         return ''
 
     security.declareProtected(permissions.View, 'isDiscussable')
+
     def isDiscussable(self, encoding=None):
+        log_deprecated(
+            "The isDiscussable method from the ExtensibleMetadata in "
+            "Products.ATContentTypes has been deprecated and will be removed "
+            "in Plone 5. This method belongs to the old discussion "
+            "infrastructure that already has been replaced by "
+            "plone.app.discussion in Plone 4.1."
+        )
+        if not 'portal_discussion' in self.objectIds():
+            return
         # Returns either True or False
         dtool = getToolByName(self, 'portal_discussion')
         return dtool.isDiscussionAllowedFor(self)
 
     security.declareProtected(permissions.View, 'editIsDiscussable')
+
     def editIsDiscussable(self, encoding=None):
+        log_deprecated(
+            "The editIsDiscussable method from the ExtensibleMetadata in "
+            "Products.ATContentTypes has been deprecated and will be removed "
+            "in Plone 5. This method belongs to the old discussion "
+            "infrastructure that already has been replaced by "
+            "plone.app.discussion in Plone 4.1."
+        )
+        if not 'portal_discussion' in self.objectIds():
+            return
         # Returns True, False or if None the default value
         result = self.rawIsDiscussable()
         if result is not None:
@@ -250,7 +276,17 @@ class ExtensibleMetadata(Persistence.Persistent):
         return default
 
     security.declareProtected(permissions.View, 'rawIsDiscussable')
+
     def rawIsDiscussable(self):
+        log_deprecated(
+            "The rawIsDiscussable method from the ExtensibleMetadata in "
+            "Products.ATContentTypes has been deprecated and will be removed "
+            "in Plone 5. This method belongs to the old discussion "
+            "infrastructure that already has been replaced by "
+            "plone.app.discussion in Plone 4.1."
+        )
+        if not 'portal_discussion' in self.objectIds():
+            return
         # Returns True, False or None where None means use the default
         result = getattr(aq_base(self), 'allow_discussion', None)
         if result is not None:
@@ -258,7 +294,17 @@ class ExtensibleMetadata(Persistence.Persistent):
         return result
 
     security.declareProtected(permissions.View, 'defaultIsDiscussable')
+
     def defaultIsDiscussable(self):
+        log_deprecated(
+            "The defaultIsDiscussable method from the ExtensibleMetadata in "
+            "Products.ATContentTypes has been deprecated and will be removed "
+            "in Plone 5. This method belongs to the old discussion "
+            "infrastructure that already has been replaced by "
+            "plone.app.discussion in Plone 4.1."
+        )
+        if not 'portal_discussion' in self.objectIds():
+            return
         # Returns the default value, either True or False
         default = None
         typeInfo = self.getTypeInfo()
@@ -268,40 +314,14 @@ class ExtensibleMetadata(Persistence.Persistent):
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'allowDiscussion')
-    def allowDiscussion(self, allowDiscussion=None, **kw):
-        default = self.defaultIsDiscussable()
-        current = self.rawIsDiscussable()
 
-        # If we already overwrote the default or the value we try to set is
-        # not the default we change it. Otherwise we keep what's there.
-        if (current is not None or
-            (current is None and default != allowDiscussion)):
-            dtool = getToolByName(self, 'portal_discussion', None)
-            try:
-                if dtool is not None:
-                    try:
-                        dtool.overrideDiscussionFor(self, allowDiscussion)
-                    except AttributeError:
-                        # CMF 2.1.0's CMFDefault.DiscussionTool
-                        # has tried to delete the class attribute.
-                        # TODO: remove this when we move to a later
-                        # CMF.
-                        pass
-            except Unauthorized:
-                # Catch Unauthorized exception that could be raised by the
-                # discussion tool when the authenticated users hasn't
-                # ModifyPortalContent permissions.
-                # Explanation:
-                # A user might have CreatePortalContent but not ModifyPortalContent
-                # so allowDiscussion could raise a Unauthorized error although it's
-                # called from trusted code. That is VERY bad inside setDefault()!
-                log('Catched Unauthorized on discussiontool.' \
-                    'overrideDiscussionFor(%s)' % self.absolute_url(1),
-                    level=DEBUG)
+    def allowDiscussion(self, allowDiscussion=None, **kw):
+        pass
 
     # Vocabulary methods ######################################################
 
     security.declareProtected(permissions.View, 'languages')
+
     def languages(self):
         """Vocabulary method for the language field
         """
@@ -324,18 +344,19 @@ class ExtensibleMetadata(Persistence.Persistent):
             # Fall back to static definition
             if languages is None:
                 return DisplayList(
-                    (('en','English'), ('fr','French'), ('es','Spanish'),
-                     ('pt','Portuguese'), ('ru','Russian')))
+                    (('en', 'English'), ('fr', 'French'), ('es', 'Spanish'),
+                     ('pt', 'Portuguese'), ('ru', 'Russian')))
         else:
             languages = util.getLanguageListing(combined=use_combined)
-            languages.sort(key=lambda x:x[1])
+            languages.sort(key=lambda x: x[1])
             # Put language neutral at the top.
-            languages.insert(0,(u'',_(u'Language neutral')))
+            languages.insert(0, (u'', _(u'Language neutral')))
         return DisplayList(languages)
 
     #  DublinCore interface query methods #####################################
 
     security.declareProtected(permissions.View, 'CreationDate')
+
     def CreationDate(self, zone=None):
         """ Dublin Core element - date resource created.
         """
@@ -345,7 +366,8 @@ class ExtensibleMetadata(Persistence.Persistent):
         # return unknown if never set properly
         return creation is None and 'Unknown' or creation.toZone(zone).ISO8601()
 
-    security.declareProtected( permissions.View, 'EffectiveDate')
+    security.declareProtected(permissions.View, 'EffectiveDate')
+
     def EffectiveDate(self, zone=None):
         """ Dublin Core element - date resource becomes effective.
         """
@@ -362,8 +384,8 @@ class ExtensibleMetadata(Persistence.Persistent):
     security.declareProtected(permissions.View, 'effective_date')
     effective_date = ComputedAttribute(_effective_date, 1)
 
+    security.declareProtected(permissions.View, 'ExpirationDate')
 
-    security.declareProtected( permissions.View, 'ExpirationDate')
     def ExpirationDate(self, zone=None):
         """Dublin Core element - date resource expires.
         """
@@ -381,6 +403,7 @@ class ExtensibleMetadata(Persistence.Persistent):
     expiration_date = ComputedAttribute(_expiration_date, 1)
 
     security.declareProtected(permissions.View, 'Date')
+
     def Date(self, zone=None):
         """
         Dublin Core element - default date
@@ -396,6 +419,7 @@ class ExtensibleMetadata(Persistence.Persistent):
                 effective.toZone(zone).ISO8601())
 
     security.declareProtected(permissions.View, 'Format')
+
     def Format(self):
         """cmf/backward compat
         Dublin Core element - resource format
@@ -405,6 +429,7 @@ class ExtensibleMetadata(Persistence.Persistent):
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'setFormat')
+
     def setFormat(self, value):
         """cmf/backward compat: ignore setFormat"""
         self.setContentType(value)
@@ -416,16 +441,18 @@ class ExtensibleMetadata(Persistence.Persistent):
     #  DublinCore utility methods #############################################
 
     security.declareProtected(permissions.View, 'contentEffective')
+
     def contentEffective(self, date):
         """Is the date within the resource's effective range?
         """
         effective = self.getField('effectiveDate').get(self)
         expires = self.getField('expirationDate').get(self)
-        pastEffective = ( effective is None or effective <= date )
-        beforeExpiration = ( expires is None or expires >= date )
+        pastEffective = (effective is None or effective <= date)
+        beforeExpiration = (expires is None or expires >= date)
         return pastEffective and beforeExpiration
 
     security.declareProtected(permissions.View, 'contentExpired')
+
     def contentExpired(self, date=None):
         """ Is the date after resource's expiration """
         if not date:
@@ -438,6 +465,7 @@ class ExtensibleMetadata(Persistence.Persistent):
     #  CatalogableDublinCore methods ##########################################
 
     security.declareProtected(permissions.View, 'created')
+
     def created(self):
         """Dublin Core element - date resource created,
         returned as DateTime.
@@ -447,6 +475,7 @@ class ExtensibleMetadata(Persistence.Persistent):
         return created is None and FLOOR_DATE or created
 
     security.declareProtected(permissions.View, 'modified')
+
     def modified(self):
         """Dublin Core element - date resource last modified,
         returned as DateTime.
@@ -456,6 +485,7 @@ class ExtensibleMetadata(Persistence.Persistent):
         return modified
 
     security.declareProtected(permissions.View, 'effective')
+
     def effective(self):
         """Dublin Core element - date resource becomes effective,
         returned as DateTime.
@@ -464,6 +494,7 @@ class ExtensibleMetadata(Persistence.Persistent):
         return effective is None and FLOOR_DATE or effective
 
     security.declareProtected(permissions.View, 'expires')
+
     def expires(self):
         """Dublin Core element - date resource expires,
         returned as DateTime.
@@ -493,6 +524,7 @@ class ExtensibleMetadata(Persistence.Persistent):
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'notifyModified')
+
     def notifyModified(self):
         """
         Take appropriate action after the resource has been modified.
@@ -504,6 +536,7 @@ class ExtensibleMetadata(Persistence.Persistent):
 
     security.declareProtected(permissions.ManagePortal,
                               'setModificationDate')
+
     def setModificationDate(self, modification_date=None):
         """Set the date when the resource was last modified.
         When called without an argument, sets the date to now.
@@ -516,6 +549,7 @@ class ExtensibleMetadata(Persistence.Persistent):
 
     security.declareProtected(permissions.ManagePortal,
                               'setCreationDate')
+
     def setCreationDate(self, creation_date=None):
         """Set the date when the resource was created.
         When called without an argument, sets the date to now.
@@ -526,7 +560,8 @@ class ExtensibleMetadata(Persistence.Persistent):
             created = self._datify(creation_date)
         self.getField('creation_date').set(self, created)
 
-    security.declarePrivate( '_datify' )
+    security.declarePrivate('_datify')
+
     def _datify(self, date):
         """Try to convert something into a DateTime instance or None
         """
@@ -542,6 +577,7 @@ class ExtensibleMetadata(Persistence.Persistent):
     #  DublinCore interface query methods
     #
     security.declareProtected(permissions.View, 'Publisher')
+
     def Publisher(self):
         """Dublin Core element - resource publisher
         """
@@ -549,6 +585,7 @@ class ExtensibleMetadata(Persistence.Persistent):
         return 'No publisher'
 
     security.declareProtected(permissions.View, 'ModificationDate')
+
     def ModificationDate(self, zone=None):
         """ Dublin Core element - date resource last modified.
         """
@@ -559,6 +596,7 @@ class ExtensibleMetadata(Persistence.Persistent):
                 or modified.toZone(zone).ISO8601())
 
     security.declareProtected(permissions.View, 'Type')
+
     def Type(self):
         """Dublin Core element - Object type"""
         if hasattr(aq_base(self), 'getTypeInfo'):
@@ -568,6 +606,7 @@ class ExtensibleMetadata(Persistence.Persistent):
         return self.meta_type
 
     security.declareProtected(permissions.View, 'Identifier')
+
     def Identifier(self):
         """Dublin Core element - Object ID"""
         # XXX: fixme using 'portal_metadata' (we need to prepend the
@@ -575,12 +614,14 @@ class ExtensibleMetadata(Persistence.Persistent):
         return self.absolute_url()
 
     security.declareProtected(permissions.View, 'listContributors')
+
     def listContributors(self):
         """Dublin Core element - Contributors"""
         return self.Contributors()
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'addCreator')
+
     def addCreator(self, creator=None):
         """ Add creator to Dublin Core creators.
         """
@@ -596,6 +637,7 @@ class ExtensibleMetadata(Persistence.Persistent):
             self.setCreators(curr_creators + (creator, ))
 
     security.declareProtected(permissions.View, 'listCreators')
+
     def listCreators(self):
         """ List Dublin Core Creator elements - resource authors.
         """
@@ -612,6 +654,7 @@ class ExtensibleMetadata(Persistence.Persistent):
         return creators.get(self)
 
     security.declareProtected(permissions.View, 'Creator')
+
     def Creator(self):
         """ Dublin Core Creator element - resource author.
         """
@@ -633,118 +676,122 @@ class ExtensibleMetadata(Persistence.Persistent):
     #
 
     security.declareProtected(permissions.View, 'getMetadataHeaders')
+
     def getMetadataHeaders(self):
         """ Return RFC-822-style headers.
         """
         hdrlist = []
-        hdrlist.append( ( 'Title', self.Title() ) )
-        hdrlist.append( ( 'Subject', string.join( self.Subject(), ', ' ) ) )
-        hdrlist.append( ( 'Publisher', self.Publisher() ) )
-        hdrlist.append( ( 'Description', self.Description() ) )
-        hdrlist.append( ( 'Contributors', string.join(
-            self.Contributors(), '; ' ) ) )
-        hdrlist.append( ( 'Creators', string.join(
-            self.Creators(), '; ' ) ) )
-        hdrlist.append( ( 'Effective_date', self.EffectiveDate() ) )
-        hdrlist.append( ( 'Expiration_date', self.ExpirationDate() ) )
-        hdrlist.append( ( 'Type', self.Type() ) )
-        hdrlist.append( ( 'Format', self.Format() ) )
-        hdrlist.append( ( 'Language', self.Language() ) )
-        hdrlist.append( ( 'Rights', self.Rights() ) )
+        hdrlist.append(('Title', self.Title()))
+        hdrlist.append(('Subject', string.join(self.Subject(), ', ')))
+        hdrlist.append(('Publisher', self.Publisher()))
+        hdrlist.append(('Description', self.Description()))
+        hdrlist.append(('Contributors', string.join(
+            self.Contributors(), '; ')))
+        hdrlist.append(('Creators', string.join(
+            self.Creators(), '; ')))
+        hdrlist.append(('Effective_date', self.EffectiveDate()))
+        hdrlist.append(('Expiration_date', self.ExpirationDate()))
+        hdrlist.append(('Type', self.Type()))
+        hdrlist.append(('Format', self.Format()))
+        hdrlist.append(('Language', self.Language()))
+        hdrlist.append(('Rights', self.Rights()))
         return hdrlist
 
     #
     #  Management tab methods
     #
 
-    security.declarePrivate( '_editMetadata' )
-    def _editMetadata(self
-                      , title=_marker
-                      , subject=_marker
-                      , description=_marker
-                      , contributors=_marker
-                      , effective_date=_marker
-                      , expiration_date=_marker
-                      , format=_marker
-                      , language=_marker
-                      , rights=_marker
+    security.declarePrivate('_editMetadata')
+
+    def _editMetadata(self,
+                      title=_marker,
+                      subject=_marker,
+                      description=_marker,
+                      contributors=_marker,
+                      effective_date=_marker,
+                      expiration_date=_marker,
+                      format=_marker,
+                      language=_marker,
+                      rights=_marker,
                       ):
         """ Update the editable metadata for this resource.
         """
         if title is not _marker:
-            self.setTitle( title )
+            self.setTitle(title)
         if subject is not _marker:
-            self.setSubject( subject )
+            self.setSubject(subject)
         if description is not _marker:
-            self.setDescription( description )
+            self.setDescription(description)
         if contributors is not _marker:
-            self.setContributors( contributors )
+            self.setContributors(contributors)
         if effective_date is not _marker:
-            self.setEffectiveDate( effective_date )
+            self.setEffectiveDate(effective_date)
         if expiration_date is not _marker:
-            self.setExpirationDate( expiration_date )
+            self.setExpirationDate(expiration_date)
         if format is not _marker:
-            self.setFormat( format )
+            self.setFormat(format)
         if language is not _marker:
-            self.setLanguage( language )
+            self.setLanguage(language)
         if rights is not _marker:
-            self.setRights( rights )
+            self.setRights(rights)
 
     security.declareProtected(permissions.ModifyPortalContent,
-                              'manage_metadata' )
-    manage_metadata = DTMLFile('zmi_metadata', _dtmldir)
+                              'manage_metadata')
+    manage_metadata = DTMLFile('zmi_metadata', config._www)
 
     security.declareProtected(permissions.ModifyPortalContent,
-                               'manage_editMetadata')
-    def manage_editMetadata( self
-                           , title
-                           , subject
-                           , description
-                           , contributors
-                           , effective_date
-                           , expiration_date
-                           , format
-                           , language
-                           , rights
-                           , REQUEST
-                           ):
+                              'manage_editMetadata')
+
+    def manage_editMetadata(self,
+                            title,
+                            subject,
+                            description,
+                            contributors,
+                            effective_date,
+                            expiration_date,
+                            format,
+                            language,
+                            rights,
+                            REQUEST,
+                            ):
         """ Update metadata from the ZMI.
         """
-        self._editMetadata( title, subject, description, contributors
-                          , effective_date, expiration_date
-                          , format, language, rights
-                          )
-        REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
-                                + '/manage_metadata'
-                                + '?manage_tabs_message=Metadata+updated.' )
+        self._editMetadata(title, subject, description, contributors,
+                           effective_date, expiration_date,
+                           format, language, rights,
+                           )
+        REQUEST['RESPONSE'].redirect(self.absolute_url()
+                                     + '/manage_metadata'
+                                     + '?manage_tabs_message=Metadata+updated.')
 
     security.declareProtected(permissions.ModifyPortalContent,
                               'editMetadata')
-    def editMetadata(self
-                     , title=''
-                     , subject=()
-                     , description=''
-                     , contributors=()
-                     , effective_date=None
-                     , expiration_date=None
-                     , format='text/html'
-                     , language='en-US'
-                     , rights=''
+
+    def editMetadata(self,
+                     title='',
+                     subject=(),
+                     description='',
+                     contributors=(),
+                     effective_date=None,
+                     expiration_date=None,
+                     format='text/html',
+                     language='en-US',
+                     rights='',
                      ):
         """
         used to be:  editMetadata = WorkflowAction(_editMetadata)
         Need to add check for webDAV locked resource for TTW methods.
         """
         self.failIfLocked()
-        self._editMetadata(title=title
-                           , subject=subject
-                           , description=description
-                           , contributors=contributors
-                           , effective_date=effective_date
-                           , expiration_date=expiration_date
-                           , format=format
-                           , language=language
-                           , rights=rights
+        self._editMetadata(title=title,
+                           subject=subject,
+                           description=description,
+                           contributors=contributors,
+                           effective_date=effective_date,
+                           expiration_date=expiration_date,
+                           format=format,
+                           language=language,
+                           rights=rights,
                            )
         self.reindexObject()
 
