@@ -6,14 +6,13 @@ from zope.interface import implements
 
 from Products.CMFCore.utils import getToolByName
 from Products.Archetypes.interfaces.referenceable import IReferenceable
+from Products.Archetypes.interfaces import IContentReference
 from Products.Archetypes.interfaces import IReference
 from Products.Archetypes.interfaces import IReferenceCatalog
-from Products.Archetypes.interfaces.referenceengine import \
-    IContentReference, IReference as Z2IReference
 
 from Products.Archetypes.utils import make_uuid, getRelURL, shasattr
 from Products.Archetypes.config import (
-    TOOL_NAME, UID_CATALOG, REFERENCE_CATALOG, UUID_ATTR)
+    TOOL_NAME, UID_CATALOG, REFERENCE_CATALOG, UUID_ATTR, _www)
 from Products.Archetypes.exceptions import ReferenceException
 
 from Acquisition import aq_base, aq_parent
@@ -29,10 +28,11 @@ from Products.CMFCore import permissions
 from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from Products.ZCatalog.ZCatalog import ZCatalog
 from Products.ZCatalog.Catalog import Catalog
+from Products.ZCatalog.Lazy import LazyMap
 from Products import CMFCore
 
+from plone.uuid.interfaces import IUUID
 
-_www = os.path.join(os.path.dirname(__file__), 'www')
 _catalog_dtml = os.path.join(os.path.dirname(CMFCore.__file__), 'dtml')
 
 STRING_TYPES = (StringType, UnicodeType)
@@ -41,13 +41,14 @@ from Referenceable import Referenceable
 from UIDCatalog import UIDCatalogBrains
 from UIDCatalog import UIDResolver
 
+
 class Reference(Referenceable, SimpleItem):
-    ## Added base level support for referencing References
-    ## They respond to the UUID protocols, but are not
-    ## catalog aware. This means that you can't move/rename
-    ## reference objects and expect them to work, but you can't
-    ## do this anyway. However they should fine the correct
-    ## events when they are added/deleted, etc
+    # Added base level support for referencing References
+    # They respond to the UUID protocols, but are not
+    # catalog aware. This means that you can't move/rename
+    # reference objects and expect them to work, but you can't
+    # do this anyway. However they should fine the correct
+    # events when they are added/deleted, etc
 
     implements(IReference)
 
@@ -57,13 +58,9 @@ class Reference(Referenceable, SimpleItem):
 
     # XXX FIXME more security
 
-    manage_options = (
-        (
-        {'label':'View', 'action':'manage_view',
-         },
-        )+
-        SimpleItem.manage_options
-        )
+    manage_options = ((
+        {'label': 'View', 'action': 'manage_view'},
+    ) + SimpleItem.manage_options)
 
     security.declareProtected(permissions.ManagePortal,
                               'manage_view')
@@ -80,34 +77,24 @@ class Reference(Referenceable, SimpleItem):
         self.__dict__.update(kwargs)
 
     def __repr__(self):
-        return "<Reference sid:%s tid:%s rel:%s>" %(self.sourceUID, self.targetUID, self.relationship)
+        return "<Reference sid:%s tid:%s rel:%s>" % (self.sourceUID,
+                                                     self.targetUID,
+                                                     self.relationship)
 
     def UID(self):
         """the uid method for compat"""
-        return getattr(aq_base(self), UUID_ATTR)
+        return IUUID(self, None)
 
-    ###
     # Convenience methods
+
     def getSourceObject(self):
-        tool = getToolByName(self, UID_CATALOG, None)
-        if tool is None: return ''
-        brains = tool(UID=self.sourceUID)
-        for brain in brains:
-            obj = brain.getObject()
-            if obj is not None:
-                return obj
+        return self._optimizedGetObject(self.sourceUID)
 
     def getTargetObject(self):
-        tool = getToolByName(self, UID_CATALOG, None)
-        if tool is None: return ''
-        brains = tool(UID=self.targetUID)
-        for brain in brains:
-            obj = brain.getObject()
-            if obj is not None:
-                return obj
+        return self._optimizedGetObject(self.targetUID)
 
-    ###
     # Catalog support
+
     def targetId(self):
         target = self.getTargetObject()
         if target is not None:
@@ -126,11 +113,11 @@ class Reference(Referenceable, SimpleItem):
     ###
     # Policy hooks, subclass away
     def addHook(self, tool, sourceObject=None, targetObject=None):
-        #to reject the reference being added raise a ReferenceException
+        # to reject the reference being added raise a ReferenceException
         pass
 
     def delHook(self, tool, sourceObject=None, targetObject=None):
-        #to reject the delete raise a ReferenceException
+        # to reject the delete raise a ReferenceException
         pass
 
     ###
@@ -158,7 +145,7 @@ class Reference(Referenceable, SimpleItem):
 
     def manage_beforeDelete(self, item, container):
         Referenceable.manage_beforeDelete(self, item, container)
-        rc  = getToolByName(container, REFERENCE_CATALOG)
+        rc = getToolByName(container, REFERENCE_CATALOG)
         url = getRelURL(container, self.getPhysicalPath())
         rc.uncatalog_object(url)
 
@@ -166,6 +153,7 @@ InitializeClass(Reference)
 
 REFERENCE_CONTENT_INSTANCE_NAME = 'content'
 
+
 class ContentReference(ObjectManager, Reference):
     '''Subclass of Reference to support contentish objects inside references '''
 
@@ -174,26 +162,25 @@ class ContentReference(ObjectManager, Reference):
     def __init__(self, *args, **kw):
         Reference.__init__(self, *args, **kw)
 
-
     security = ClassSecurityInfo()
     # XXX FIXME more security
 
     def addHook(self, *args, **kw):
         # creates the content instance
-        if type(self.contentType) in (type(''),type(u'')):
+        if type(self.contentType) in (type(''), type(u'')):
             # type given as string
-            tt=getToolByName(self,'portal_types')
+            tt = getToolByName(self, 'portal_types')
             tt.constructContent(self.contentType, self,
                                 REFERENCE_CONTENT_INSTANCE_NAME)
         else:
             # type given as class
             setattr(self, REFERENCE_CONTENT_INSTANCE_NAME,
                     self.contentType(REFERENCE_CONTENT_INSTANCE_NAME))
-            getattr(self, REFERENCE_CONTENT_INSTANCE_NAME)._md=PersistentMapping()
+            getattr(self, REFERENCE_CONTENT_INSTANCE_NAME)._md = PersistentMapping()
 
     def delHook(self, *args, **kw):
         # remove the content instance
-        if type(self.contentType) in (type(''),type(u'')):
+        if type(self.contentType) in (type(''), type(u'')):
             # type given as string
             self._delObject(REFERENCE_CONTENT_INSTANCE_NAME)
         else:
@@ -213,26 +200,27 @@ class ContentReference(ObjectManager, Reference):
 
 InitializeClass(ContentReference)
 
+
 class ContentReferenceCreator:
     '''Helper class to construct ContentReference instances based
        on a certain content type '''
 
     security = ClassSecurityInfo()
 
-    def __init__(self,contentType):
-        self.contentType=contentType
+    def __init__(self, contentType):
+        self.contentType = contentType
 
-    def __call__(self,*args,**kw):
-        #simulates the constructor call to the reference class in addReference
-        res=ContentReference(*args,**kw)
-        res.contentType=self.contentType
+    def __call__(self, *args, **kw):
+        # simulates the constructor call to the reference class in addReference
+        res = ContentReference(*args, **kw)
+        res.contentType = self.contentType
 
         return res
 
 InitializeClass(ContentReferenceCreator)
 
-# The brains we want to use
 
+# The brains we want to use
 class ReferenceCatalogBrains(UIDCatalogBrains):
     pass
 
@@ -254,9 +242,9 @@ class PluggableCatalog(Catalog):
         schema = self.schema
         scopy = schema.copy()
 
-        scopy['data_record_id_']=len(schema.keys())
-        scopy['data_record_score_']=len(schema.keys())+1
-        scopy['data_record_normalized_score_']=len(schema.keys())+2
+        scopy['data_record_id_'] = len(schema.keys())
+        scopy['data_record_score_'] = len(schema.keys()) + 1
+        scopy['data_record_normalized_score_'] = len(schema.keys()) + 2
 
         plugbrains.__record_schema__ = scopy
 
@@ -265,6 +253,7 @@ class PluggableCatalog(Catalog):
 
 InitializeClass(PluggableCatalog)
 
+
 class ReferenceBaseCatalog(PluggableCatalog):
     BASE_CLASS = ReferenceCatalogBrains
 
@@ -272,6 +261,7 @@ class ReferenceBaseCatalog(PluggableCatalog):
 class IndexableObjectWrapper(object):
     """Wwrapper for object indexing
     """
+
     def __init__(self, obj):
         self._obj = obj
 
@@ -307,7 +297,7 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
         self._catalog = ReferenceBaseCatalog()
 
     ###
-    ## Public API
+    # Public API
     def addReference(self, source, target, relationship=None,
                      referenceClass=None, updateReferences=True, **kwargs):
         sID, sobj = self._uidFor(source)
@@ -322,7 +312,7 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
             objects = self._resolveBrains(
                 self._queryFor(sID, tID, relationship))
             if objects:
-                #we want to update the existing reference
+                # we want to update the existing reference
                 existing = objects[0]
                 if existing:
                     # We can't del off self, we now need to remove it
@@ -330,7 +320,6 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
                     annotation = sobj._getReferenceAnnotations()
                     annotation._delObject(existing.id)
 
-
         rID = self._makeName(sID, tID)
         if not referenceClass:
             referenceClass = Reference
@@ -367,29 +356,82 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
         for b in self.getBackReferences(object, relationship):
             self._deleteReference(b)
 
-    def getReferences(self, object, relationship=None, targetObject=None):
+    security.declarePrivate('getReferences')
+
+    def getReferences(self, object, relationship=None, targetObject=None,
+                      objects=True):
         # return a collection of reference objects
-        sID, sobj = self._uidFor(object)
-        if targetObject:
-            tID, tobj = self._uidFor(targetObject)
-        else:
-            tID, tobj = None,None
+        return self._optimizedReferences(object, relationship=relationship,
+                                         targetObject=targetObject, objects=objects, attribute='sourceUID')
 
-        brains = self._queryFor(sid=sID, relationship=relationship, tid=tID)
-        return self._resolveBrains(brains)
+    security.declarePrivate('getBackReferences')
 
-    def getBackReferences(self, object, relationship=None, targetObject=None):
+    def getBackReferences(self, object, relationship=None, targetObject=None,
+                          objects=True):
         # return a collection of reference objects
 
         # Back refs would be anything that target this object
+        return self._optimizedReferences(object, relationship=relationship,
+                                         targetObject=targetObject, objects=objects, attribute='targetUID')
+
+    def _optimizedReferences(self, object, relationship=None,
+                             targetObject=None, objects=True, attribute='sourceUID'):
+
         sID, sobj = self._uidFor(object)
         if targetObject:
             tID, tobj = self._uidFor(targetObject)
+            if attribute == 'sourceUID':
+                brains = self._queryFor(sID, tID, relationship)
+            else:
+                brains = self._queryFor(tID, sID, relationship)
         else:
-            tID, tobj = None,None
+            brains = self._optimizedQuery(sID, attribute, relationship)
 
-        brains = self._queryFor(tid=sID, relationship=relationship, sid=tID)
-        return self._resolveBrains(brains)
+        if objects:
+            return self._resolveBrains(brains)
+        return brains
+
+    def _optimizedQuery(self, uid, indexname, relationship):
+        """query reference catalog for object matching the info we are
+        given, returns brains
+        """
+        if not uid:  # pragma: no cover
+            return []
+
+        _catalog = self._catalog
+        indexes = _catalog.indexes
+
+        # First get one or multiple record ids for the source/target uid index
+        rids = indexes[indexname]._index.get(uid, None)
+        if rids is None:
+            return []
+        elif isinstance(rids, int):
+            rids = [rids]
+        else:
+            rids = list(rids)
+
+        # As a second step make sure we only get references of the right type
+        # The unindex holds data of the type: [(-311870037, 'relatesTo')]
+        # The index holds data like: [('relatesTo', -311870037)]
+        if relationship is None:
+            result_rids = rids
+        else:
+            rel_unindex_get = indexes['relationship']._unindex.get
+            result_rids = set()
+            if isinstance(relationship, str):
+                relationship = set([relationship])
+            for r in rids:
+                rels = rel_unindex_get(r, None)
+                if rels is None:
+                    rels = set()
+                elif isinstance(rels, str):
+                    rels = set([rels])
+                if not rels.isdisjoint(relationship):
+                    result_rids.add(r)
+
+        # Create brains
+        return LazyMap(_catalog.__getitem__,
+                       list(result_rids), len(result_rids))
 
     def hasRelationshipTo(self, source, target, relationship):
         sID, sobj = self._uidFor(source)
@@ -418,11 +460,10 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
         brains = self._queryFor(tid=sID)
         res = {}
         for b in brains:
-            res[b.relationship]=1
+            res[b.relationship] = 1
 
         return res.keys()
 
-
     def isReferenceable(self, object):
         return (IReferenceable.providedBy(object) or
                 shasattr(object, 'isReferenceable'))
@@ -441,20 +482,23 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
             return obj
 
     #####
-    ## UID register/unregister
-    security.declareProtected(permissions.ModifyPortalContent, 'registerObject')
+    # UID register/unregister
+    security.declareProtected(
+        permissions.ModifyPortalContent, 'registerObject')
+
     def registerObject(self, object):
         self._uidFor(object)
 
-    security.declareProtected(permissions.ModifyPortalContent, 'unregisterObject')
+    security.declareProtected(
+        permissions.ModifyPortalContent, 'unregisterObject')
+
     def unregisterObject(self, object):
         self.deleteReferences(object)
         uc = getToolByName(self, UID_CATALOG)
         uc.uncatalog_object(object._getURL())
 
-
     ######
-    ## Private/Internal
+    # Private/Internal
     def _objectByUUID(self, uuid):
         tool = getToolByName(self, UID_CATALOG)
         brains = tool(UID=uuid)
@@ -474,15 +518,18 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
         """
 
         query = {}
-        if sid: query['sourceUID'] = sid
-        if tid: query['targetUID'] = tid
-        if relationship: query['relationship'] = relationship
-        if targetId: query['targetId'] = targetId
+        if sid:
+            query['sourceUID'] = sid
+        if tid:
+            query['targetUID'] = tid
+        if relationship:
+            query['relationship'] = relationship
+        if targetId:
+            query['targetId'] = targetId
         brains = self.searchResults(query, merge=merge)
 
         return brains
 
-
     def _uidFor(self, obj):
         # We should really check for the interface but I have an idea
         # about simple annotated objects I want to play out
@@ -491,15 +538,14 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
             if not self.isReferenceable(uobject):
                 raise ReferenceException, "%r not referenceable" % uobject
 
-            # shasattr() doesn't work here
-            if not getattr(aq_base(uobject), UUID_ATTR, None):
+            uuid = IUUID(uobject, None)
+            if uuid is None:
                 uuid = self._getUUIDFor(uobject)
-            else:
-                uuid = getattr(uobject, UUID_ATTR)
+
         else:
             uuid = obj
             obj = None
-            #and we look up the object
+            # and we look up the object
             uid_catalog = getToolByName(self, UID_CATALOG)
             brains = uid_catalog(dict(UID=uuid))
             for brain in brains:
@@ -525,7 +571,7 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
         else:
             annotation = sobj._getReferenceAnnotations()
             try:
-                annotation._delObject(referenceObject.UID())
+                annotation._delObject(IUUID(referenceObject, None))
             except (AttributeError, KeyError):
                 pass
 
@@ -544,49 +590,45 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
     def __nonzero__(self):
         return 1
 
-    def _catalogReferencesFor(self,obj,path):
+    def _catalogReferencesFor(self, obj, path):
         if IReferenceable.providedBy(obj):
             obj._catalogRefs(self)
 
-    def _catalogReferences(self,root=None,**kw):
+    def _catalogReferences(self, root=None, **kw):
         ''' catalogs all references, where the optional parameter 'root'
            can be used to specify the tree that has to be searched for references '''
 
         if not root:
-            root=getToolByName(self,'portal_url').getPortalObject()
+            root = getToolByName(self, 'portal_url').getPortalObject()
 
         path = '/'.join(root.getPhysicalPath())
 
-        results = self.ZopeFindAndApply(root,
-                                        search_sub=1,
-                                        apply_func=self._catalogReferencesFor,
-                                        apply_path=path,**kw)
+        self.ZopeFindAndApply(root, search_sub=1,
+                              apply_func=self._catalogReferencesFor,
+                              apply_path=path, **kw)
 
+    security.declareProtected(permissions.ManagePortal,
+                              'manage_catalogFoundItems')
 
-
-    security.declareProtected(permissions.ManagePortal, 'manage_catalogFoundItems')
     def manage_catalogFoundItems(self, REQUEST, RESPONSE, URL2, URL1,
                                  obj_metatypes=None,
                                  obj_ids=None, obj_searchterm=None,
                                  obj_expr=None, obj_mtime=None,
                                  obj_mspec=None, obj_roles=None,
                                  obj_permission=None):
-
         """ Find object according to search criteria and Catalog them
         """
 
-
         elapse = time.time()
         c_elapse = time.clock()
 
-        words = 0
         obj = REQUEST.PARENTS[1]
 
-        self._catalogReferences(obj,obj_metatypes=obj_metatypes,
-                                 obj_ids=obj_ids, obj_searchterm=obj_searchterm,
-                                 obj_expr=obj_expr, obj_mtime=obj_mtime,
-                                 obj_mspec=obj_mspec, obj_roles=obj_roles,
-                                 obj_permission=obj_permission)
+        self._catalogReferences(obj, obj_metatypes=obj_metatypes,
+                                obj_ids=obj_ids, obj_searchterm=obj_searchterm,
+                                obj_expr=obj_expr, obj_mtime=obj_mtime,
+                                obj_mspec=obj_mspec, obj_roles=obj_roles,
+                                obj_permission=obj_permission)
 
         elapse = time.time() - elapse
         c_elapse = time.clock() - c_elapse
@@ -598,9 +640,11 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
                          'Total time: %s\n'
                          'Total CPU time: %s'
                          % (`elapse`, `c_elapse`))
-            )
+        )
+
+    security.declareProtected(
+        permissions.ManagePortal, 'manage_rebuildCatalog')
 
-    security.declareProtected(permissions.ManagePortal, 'manage_rebuildCatalog')
     def manage_rebuildCatalog(self, REQUEST=None, RESPONSE=None):
         """
         """
@@ -628,26 +672,25 @@ class ReferenceCatalog(UniqueObject, UIDResolver, ZCatalog):
 
         if RESPONSE:
             RESPONSE.redirect(
-            REQUEST.URL1 +
-            '/manage_catalogView?manage_tabs_message=' +
-            urllib.quote('Catalog Rebuilded\n'
-                         'Total time: %s\n'
-                         'Total CPU time: %s'
-                         % (`elapse`, `c_elapse`))
+                REQUEST.URL1 +
+                '/manage_catalogView?manage_tabs_message=' +
+                urllib.quote('Catalog Rebuilded\n'
+                             'Total time: %s\n'
+                             'Total CPU time: %s'
+                             % (`elapse`, `c_elapse`))
             )
 
 InitializeClass(ReferenceCatalog)
 
 
 def manage_addReferenceCatalog(self, id, title,
-                               vocab_id=None, # Deprecated
+                               vocab_id=None,  # Deprecated
                                REQUEST=None):
     """Add a ReferenceCatalog object
     """
-    id=str(id)
-    title=str(title)
-    c=ReferenceCatalog(id, title, vocab_id, self)
+    id = str(id)
+    title = str(title)
+    c = ReferenceCatalog(id, title, vocab_id, self)
     self._setObject(id, c)
     if REQUEST is not None:
-        return self.manage_main(self, REQUEST,update_menu=1)
-
+        return self.manage_main(self, REQUEST, update_menu=1)
