@@ -10,10 +10,7 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-""" CMFCore tool interfaces.
-
-$Id: _tools.py 110577 2010-04-07 06:33:17Z jens $
-"""
+""" CMFCore tool interfaces. """
 
 from zope.interface import Interface
 from zope.interface import Attribute
