@@ -10,10 +10,7 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""CMFDefault upgrade steps.
-
-$Id: __init__.py 110659 2010-04-08 15:54:42Z tseaver $
-"""
+"""CMFDefault upgrade steps. """
 
 def mark_as_finished(tool):
     """empty upgrade step handler
