@@ -11,8 +11,6 @@
 #
 ##############################################################################
 """Local component registry export / import handler.
-
-$Id: components.py 110425 2010-04-01 17:19:14Z tseaver $
 """
 
 from operator import itemgetter
@@ -24,14 +22,18 @@ from zope.component import getUtilitiesFor
 from zope.component import queryMultiAdapter
 from zope.component.interfaces import ComponentLookupError
 from zope.component.interfaces import IComponentRegistry
-from zope.location.interfaces import IPossibleSite
+try:
+    from zope.component.interfaces import IPossibleSite
+except ImportError:
+    # BBB: for Zope < 2.13 (zope.component < 3.8)
+    from zope.location.interfaces import IPossibleSite
 
 from Products.GenericSetup.interfaces import IBody
 from Products.GenericSetup.interfaces import IComponentsHandlerBlacklist
 from Products.GenericSetup.interfaces import ISetupEnviron
-from Products.GenericSetup.utils import XMLAdapterBase
 from Products.GenericSetup.utils import _getDottedName
 from Products.GenericSetup.utils import _resolveDottedName
+from Products.GenericSetup.utils import XMLAdapterBase
 
 BLACKLIST_SELF = _getDottedName(IComponentsHandlerBlacklist)
 
@@ -48,7 +50,7 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
     name = 'componentregistry'
 
     def _constructBlacklist(self):
-        blacklist = set((BLACKLIST_SELF, ))
+        blacklist = set((BLACKLIST_SELF,))
         utils = getUtilitiesFor(IComponentsHandlerBlacklist)
         for _, util in utils:
             names = [_getDottedName(i) for i in util.getExcludedInterfaces()]
@@ -197,8 +199,8 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
 
             handler = child.getAttribute('handler')
             if handler:
-                raise ValueError, "Can not specify both a factory and a " \
-                                  "handler in a subscriber registration."
+                raise ValueError("Can not specify both a factory and a "
+                                 "handler in a subscriber registration.")
 
             factory = _resolveDottedName(factory)
 
@@ -237,12 +239,12 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
 
             factory = child.getAttribute('factory')
             if factory:
-                raise ValueError, "Can not specify both a factory and a " \
-                                  "handler in a subscriber registration."
-            
+                raise ValueError("Can not specify both a factory and a "
+                                 "handler in a subscriber registration.")
+
             if child.hasAttribute('provides'):
-                raise ValueError, "Cannot use handler with provides " \
-                                  "in a subscriber registration."
+                raise ValueError("Cannot use handler with provides in a "
+                                 "subscriber registration.")
 
             handler = _resolveDottedName(handler)
 
@@ -296,13 +298,13 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
                     ofs_id = self._ofs_id(child)
                     if ofs_id in self.context.objectIds():
                         self.context._delObject(ofs_id, suppress_events=True)
-                    self.context.unregisterUtility(provided=provided, name=name)
+                    self.context.unregisterUtility(provided=provided,
+                                                   name=name)
                 continue
 
             if component and factory:
-                raise ValueError, "Can not specify both a factory and a " \
-                                  "component in a utility registration."
-
+                raise ValueError("Can not specify both a factory and a "
+                                 "component in a utility registration.")
 
             obj_path = child.getAttribute('object')
             if not component and not factory and obj_path is not None:
@@ -328,10 +330,10 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
                 self.context.registerUtility(component, provided, name)
             elif factory:
                 current = [ utility for utility in current_utilities
-                                    if utility.provided==provided and
-                                       utility.name==name ]
+                            if utility.provided == provided and
+                               utility.name == name ]
 
-                if current and getattr(current[0], "factory", None)==factory:
+                if current and getattr(current[0], "factory", None) == factory:
                     continue
 
                 obj = factory()
@@ -396,8 +398,8 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
 
         registrations = [ {'factory': _getDottedName(reg.factory),
                            'provided': _getDottedName(reg.provided),
-                           'required': reg.required}
-                          for reg in self.context.registeredSubscriptionAdapters() ]
+                           'required': reg.required} for reg
+                            in self.context.registeredSubscriptionAdapters() ]
         registrations.sort(key=itemgetter('factory'))
         registrations.sort(key=itemgetter('provided'))
         blacklist = self._constructBlacklist()
@@ -472,8 +474,8 @@ class ComponentRegistryXMLAdapter(XMLAdapterBase):
             else:
                 factory = None
                 comp = reg_info['component']
-                # check if the component is acquisition wrapped. If it is, export
-                # an object reference instead of a factory reference
+                # check if the component is acquisition wrapped. If it is,
+                # export an object reference instead of a factory reference
                 if getattr(comp, 'aq_base', None) is not None:
                     if aq_base(comp) is site:
                         child.setAttribute('object', '')
@@ -527,6 +529,7 @@ def importComponentRegistry(context):
             logger = context.getLogger('componentregistry')
             logger.debug("Nothing to import")
 
+
 def exportComponentRegistry(context):
     """Export local components.
     """
@@ -551,4 +554,3 @@ def exportComponentRegistry(context):
         if body is not None:
             context.writeDataFile('componentregistry.xml', body,
                                   exporter.mime_type)
-
