@@ -13,9 +13,10 @@
 ##############################################################################
 """ Classes: ZODBUserManager
 
-$Id: ZODBUserManager.py 113213 2010-06-06 16:23:10Z hannosch $
+$Id$
 """
 import copy
+import logging
 try:
     from hashlib import sha1 as sha
 except:
@@ -44,6 +45,9 @@ from Products.PluggableAuthService.permissions import SetOwnPassword
 from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
 from Products.PluggableAuthService.utils import classImplements
 from Products.PluggableAuthService.utils import createViewName
+from Products.PluggableAuthService.utils import csrf_only
+
+logger = logging.getLogger('PluggableAuthService')
 
 
 class IZODBUserManager(Interface):
@@ -102,13 +106,27 @@ class ZODBUserManager( BasePlugin, Cacheable ):
         if login is None or password is None:
             return None
 
-        userid = self._login_to_userid.get( login, login )
+        # Do we have a link between login and userid?  Do NOT fall
+        # back to using the login as userid when there is no match, as
+        # that gives a high chance of seeming to log in successfully,
+        # but in reality failing.
+        userid = self._login_to_userid.get(login)
+        if userid is None:
+            # Someone may be logging in with a userid instead of a
+            # login name and the two are not the same.  We could try
+            # turning those around, but really we should just fail.
+            #
+            # userid = login
+            # login = self._userid_to_login.get(userid)
+            # if login is None:
+            #     return None
+            return None
 
         reference = self._user_passwords.get(userid)
 
         if reference is None:
             return None
-        
+
         if AuthEncoding.is_encrypted( reference ):
             if AuthEncoding.pw_validate( reference, password ):
                 return userid, login
@@ -205,7 +223,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
                        , 'login' : self._userid_to_login[ user_id ]
                        , 'pluginid' : plugin_id
                        , 'editurl' : '%s?%s' % (e_url, qs)
-                       } 
+                       }
 
                 if not user_filter or user_filter( info ):
                     user_info.append( info )
@@ -253,7 +271,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
 
         o Return one mapping per user, with the following keys:
 
-          - 'user_id' 
+          - 'user_id'
           - 'login_name'
         """
         return [ self.getUserInfo( x ) for x in self._user_passwords.keys() ]
@@ -299,9 +317,72 @@ class ZODBUserManager( BasePlugin, Cacheable ):
         # The following raises a KeyError if the user_id is invalid
         old_login = self.getLoginForUserId(user_id)
 
-        del self._login_to_userid[old_login]
-        self._login_to_userid[login_name] = user_id
-        self._userid_to_login[user_id] = login_name
+        if old_login != login_name:
+
+            if self._login_to_userid.get(login_name) is not None:
+                raise ValueError('Login name not available: %s' % login_name)
+
+            del self._login_to_userid[old_login]
+            self._login_to_userid[login_name] = user_id
+            self._userid_to_login[user_id] = login_name
+        # Signal success.
+        return True
+
+    security.declarePrivate('updateEveryLoginName')
+    def updateEveryLoginName(self, quit_on_first_error=True):
+        # Update all login names to their canonical value.  This
+        # should be done after changing the login_transform property
+        # of pas.  You can set quit_on_first_error to False to report
+        # all errors before quitting with an error.  This can be
+        # useful if you want to know how many problems there are, if
+        # any.
+        pas = self._getPAS()
+        transform = pas._get_login_transform_method()
+        if not transform:
+            logger.warn("PAS has a non-existing, empty or wrong "
+                        "login_transform property.")
+            return
+
+        # Make a fresh mapping, as we do not want to add or remove
+        # items to the original mapping while we are iterating over
+        # it.
+        new_login_to_userid = OOBTree()
+        errors = []
+        for old_login_name, user_id in self._login_to_userid.items():
+            new_login_name = transform(old_login_name)
+            if new_login_name in new_login_to_userid:
+                logger.error("User id %s: login name %r already taken.",
+                             user_id, new_login_name)
+                errors.append(new_login_name)
+                if quit_on_first_error:
+                    break
+            new_login_to_userid[new_login_name] = user_id
+            if new_login_name != old_login_name:
+                self._userid_to_login[user_id] = new_login_name
+                # Also, remove from the cache
+                view_name = createViewName('enumerateUsers', user_id)
+                self.ZCacheable_invalidate(view_name=view_name)
+                logger.debug("User id %s: changed login name from %r to %r.",
+                             user_id, old_login_name, new_login_name)
+
+        # If there were errors, we do not want to save any changes.
+        if errors:
+            logger.error("There were %d errors when updating login names. "
+                         "quit_on_first_error was %r", len(errors),
+                         quit_on_first_error)
+            # Make sure the exception we raise is not swallowed.
+            self._dont_swallow_my_exceptions = True
+            raise ValueError("Transformed login names are not unique: %s." %
+                             ', '.join(errors))
+
+        # Make sure we did not lose any users.
+        assert(len(self._login_to_userid.keys())
+               == len(new_login_to_userid.keys()))
+        # Empty the main cache.
+        view_name = createViewName('enumerateUsers')
+        self.ZCacheable_invalidate(view_name=view_name)
+        # Store the new login mapping.
+        self._login_to_userid = new_login_to_userid
 
     security.declarePrivate( 'removeUser' )
     def removeUser( self, user_id ):
@@ -344,7 +425,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
     #
     #   ZMI
     #
-    manage_options = ( ( { 'label': 'Users', 
+    manage_options = ( ( { 'label': 'Users',
                            'action': 'manage_users', }
                          ,
                        )
@@ -365,12 +446,15 @@ class ZODBUserManager( BasePlugin, Cacheable ):
                                    )
 
     security.declareProtected( ManageUsers, 'manage_addUser' )
+    @csrf_only
+    @postonly
     def manage_addUser( self
                       , user_id
                       , login_name
                       , password
                       , confirm
                       , RESPONSE=None
+                      , REQUEST=None
                       ):
         """ Add a user via the ZMI.
         """
@@ -378,7 +462,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
             message = 'password+and+confirm+do+not+match'
 
         else:
-        
+
             if not login_name:
                 login_name = user_id
 
@@ -394,6 +478,8 @@ class ZODBUserManager( BasePlugin, Cacheable ):
                              )
 
     security.declareProtected( ManageUsers, 'manage_updateUserPassword' )
+    @csrf_only
+    @postonly
     def manage_updateUserPassword( self
                                  , user_id
                                  , password
@@ -407,7 +493,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
             message = 'password+and+confirm+do+not+match'
 
         else:
-        
+
             self.updateUserPassword( user_id, password )
 
             message = 'password+updated'
@@ -416,10 +502,16 @@ class ZODBUserManager( BasePlugin, Cacheable ):
             RESPONSE.redirect( '%s/manage_users?manage_tabs_message=%s'
                              % ( self.absolute_url(), message )
                              )
-    manage_updateUserPassword = postonly(manage_updateUserPassword)
 
     security.declareProtected( ManageUsers, 'manage_updateUser' )
-    def manage_updateUser(self, user_id, login_name, RESPONSE=None):
+    @csrf_only
+    @postonly
+    def manage_updateUser(self
+                         , user_id
+                         , login_name
+                         , RESPONSE=None
+                         , REQUEST=None
+                         ):
         """ Update a user's login name via the ZMI.
         """
         if not login_name:
@@ -437,6 +529,8 @@ class ZODBUserManager( BasePlugin, Cacheable ):
                              )
 
     security.declareProtected( ManageUsers, 'manage_removeUsers' )
+    @csrf_only
+    @postonly
     def manage_removeUsers( self
                           , user_ids
                           , RESPONSE=None
@@ -450,7 +544,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
             message = 'no+users+selected'
 
         else:
-        
+
             for user_id in user_ids:
                 self.removeUser( user_id )
 
@@ -460,7 +554,6 @@ class ZODBUserManager( BasePlugin, Cacheable ):
             RESPONSE.redirect( '%s/manage_users?manage_tabs_message=%s'
                              % ( self.absolute_url(), message )
                              )
-    manage_removeUsers = postonly(manage_removeUsers)
 
     #
     #   Allow users to change their own login name and password.
@@ -481,6 +574,8 @@ class ZODBUserManager( BasePlugin, Cacheable ):
                                    )
 
     security.declareProtected( SetOwnPassword, 'manage_updatePassword' )
+    @csrf_only
+    @postonly
     def manage_updatePassword( self
                              , login_name
                              , password
@@ -495,7 +590,7 @@ class ZODBUserManager( BasePlugin, Cacheable ):
             message = 'password+and+confirm+do+not+match'
 
         else:
-        
+
             if not login_name:
                 login_name = user_id
 
@@ -510,7 +605,6 @@ class ZODBUserManager( BasePlugin, Cacheable ):
                                '?manage_tabs_message=%s'
                              % ( self.absolute_url(), message )
                              )
-    manage_updatePassword = postonly(manage_updatePassword)
 
 classImplements( ZODBUserManager
                , IZODBUserManager
